package cn.wise.sc.energy.power.plant.business.task;

import cn.wise.sc.energy.power.plant.business.config.MyApplicationContextAware;
import cn.wise.sc.energy.power.plant.business.task.schedule.ScheduleUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Component;

import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Component
@ServerEndpoint("/index/realTimeData/{deviceId}")
@DependsOn("myApplicationContextAware")
public class IndexRealTimeWebSocket implements SendWebSocket{

    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;
    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    public static ConcurrentHashMap<String, IndexRealTimeWebSocket> webSocketMap = new ConcurrentHashMap<>();
    public static ConcurrentHashMap<String, String> deviceMap = new ConcurrentHashMap<>();
    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;
    /**
     * 接收  建立连接的参数信息
     */
    private String uuid;
    private IndexRealTimeTask indexRealTimeTask = (IndexRealTimeTask) MyApplicationContextAware.getApplicationContext().getBean("indexRealTimeTask");


    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("deviceId") String deviceId) {
        this.session = session;
        uuid = UUID.randomUUID().toString();
        //如果是total 表示全场
        deviceMap.put(uuid, deviceId);

        webSocketMap.put(uuid, this);
        //加入set中
        addOnlineCount();
        //启动任务
        ScheduleUtil.start(indexRealTimeTask, 1000);
        //已有用户的情况,主动推送一次
        if (webSocketMap.size() > 1) {
            indexRealTimeTask.run();
        }
        log.info("用户连接:" + deviceId + ",当前在线人数为:" + getOnlineCount());
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        if (webSocketMap.containsKey(uuid)) {
            webSocketMap.remove(uuid);
            deviceMap.remove(uuid);
            //从set中删除
            subOnlineCount();
        }
        if (webSocketMap.size() == 0) {
            ScheduleUtil.cancel(indexRealTimeTask);
        }
        log.info("用户退出:" + uuid + ",当前在线人数为:" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException {
        log.info("用户消息:" + uuid + ",报文:" + message);
        //可以群发消息
        //消息保存到数据库、redis
        if (StringUtils.isNotBlank(message)) {
            webSocketMap.get(uuid).sendMessage("111111111");
        }
    }

    /**
     * 错误
     *
     * @param session 会话
     * @param error   错误异常
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("用户错误:" + this.uuid + ",原因:" + error.getMessage());
        error.printStackTrace();
    }

    /**
     * 实现服务器主动推送
     */
    @Override
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }


    /**
     * 发送自定义消息
     *
     * @param message 消息
     * @param userId  用户
     * @throws IOException 异常
     */
    public static void sendInfo(String message, @PathParam("userId") String userId) throws IOException {
        log.info("发送消息到:" + userId + "，报文:" + message);
        if (StringUtils.isNotBlank(userId) && webSocketMap.containsKey(userId)) {
            webSocketMap.get(userId).sendMessage(message);
        } else {
            log.error("用户" + userId + ",不在线！");
        }
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        IndexRealTimeWebSocket.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        IndexRealTimeWebSocket.onlineCount--;
    }
}
